function guiEditor() {
	this.previewDoc = null;
	
	return this;
}

function voidIt(event) {
	event.stopPropagation();
	event.preventDefault();
}

guiEditor.prototype.load = function (headCode, postCode, insertCode) {
	this.previewDoc = document.getElementById("post-preview").contentDocument;

	var postCode = postCode.replace(/\n/g, "<br />");
	var insertCode = '<div id="inserted-by-scribefire">' + insertCode + '</div>';
	
	this.previewDoc.getElementsByTagName("head")[0].innerHTML = headCode;
	this.previewDoc.body.innerHTML =  postCode + insertCode;
	
	this.previewDoc.addEventListener("click", voidIt, true);
	this.previewDoc.addEventListener("mousedown", voidIt, true);
	this.previewDoc.addEventListener("mouseup", voidIt, true);
};

guiEditor.prototype.dialogAccept = function () {
	this.previewDoc.getElementById("inserted-by-scribefire").removeAttribute("id");
	
	var code = this.previewDoc.body.innerHTML;
	window.opener.setTimeout(window.opener.SF_QUICKBLOG.editPost, 100, code);
	
	return true;
};

guiEditor.prototype.moveUp = function () {
	var toMove = this.previewDoc.getElementById("inserted-by-scribefire");
	var putBefore = node_before(toMove);
	
	if (putBefore) {
		toMove.parentNode.removeChild(toMove);
		putBefore.parentNode.insertBefore(toMove, putBefore);
	}
};

guiEditor.prototype.moveDown = function () {
	var toMove = this.previewDoc.getElementById("inserted-by-scribefire");
	var putBefore = node_after(node_after(toMove));
	
	toMove.parentNode.removeChild(toMove);
	
	if (putBefore) {
		putBefore.parentNode.insertBefore(toMove, putBefore);
	}
	else {
		this.previewDoc.body.appendChild(toMove);
	}
};

guiEditor.prototype.floatLeft = function () {
	this.previewDoc.getElementById("inserted-by-scribefire").style.cssFloat = "left";
};

guiEditor.prototype.floatRight = function () {
	this.previewDoc.getElementById("inserted-by-scribefire").style.cssFloat = "right";
};

guiEditor.prototype.noFloat = function() {
	this.previewDoc.getElementById("inserted-by-scribefire").style.cssFloat = "none";
};

guiEditor.prototype.alignLeft = function () {
	this.previewDoc.getElementById("inserted-by-scribefire").style.textAlign = "left";
};

guiEditor.prototype.alignCenter = function () {
	this.previewDoc.getElementById("inserted-by-scribefire").style.textAlign = "center";
};

guiEditor.prototype.alignRight = function() {
	this.previewDoc.getElementById("inserted-by-scribefire").style.textAlign = "right";
};

/**
 * Throughout, whitespace is defined as one of the characters
 *  "\t" TAB \u0009
 *  "\n" LF  \u000A
 *  "\r" CR  \u000D
 *  " "  SPC \u0020
 *
 * This does not use Javascript's "\s" because that includes non-breaking
 * spaces (and also some other characters).
 */


/**
 * Determine whether a node's text content is entirely whitespace.
 *
 * @param nod  A node implementing the |CharacterData| interface (i.e.,
 *             a |Text|, |Comment|, or |CDATASection| node
 * @return     True if all of the text content of |nod| is whitespace,
 *             otherwise false.
 */
function is_all_ws( nod )
{
  // Use ECMA-262 Edition 3 String and RegExp features
  return !(/[^\t\n\r ]/.test(nod.data));
}


/**
 * Determine if a node should be ignored by the iterator functions.
 *
 * @param nod  An object implementing the DOM1 |Node| interface.
 * @return     true if the node is:
 *                1) A |Text| node that is all whitespace
 *                2) A |Comment| node
 *             and otherwise false.
 */

function is_ignorable( nod )
{
  return ( nod.nodeType == 8) || // A comment node
         ( (nod.nodeType == 3) && is_all_ws(nod) ); // a text node, all ws
}

/**
 * Version of |previousSibling| that skips nodes that are entirely
 * whitespace or comments.  (Normally |previousSibling| is a property
 * of all DOM nodes that gives the sibling node, the node that is
 * a child of the same parent, that occurs immediately before the
 * reference node.)
 *
 * @param sib  The reference node.
 * @return     Either:
 *               1) The closest previous sibling to |sib| that is not
 *                  ignorable according to |is_ignorable|, or
 *               2) null if no such node exists.
 */
function node_before( sib )
{
  while ((sib = sib.previousSibling)) {
    if (!is_ignorable(sib)) return sib;
  }
  return null;
}

/**
 * Version of |nextSibling| that skips nodes that are entirely
 * whitespace or comments.
 *
 * @param sib  The reference node.
 * @return     Either:
 *               1) The closest next sibling to |sib| that is not
 *                  ignorable according to |is_ignorable|, or
 *               2) null if no such node exists.
 */
function node_after( sib )
{
  while ((sib = sib.nextSibling)) {
    if (!is_ignorable(sib)) return sib;
  }
  return null;
}

/**
 * Version of |lastChild| that skips nodes that are entirely
 * whitespace or comments.  (Normally |lastChild| is a property
 * of all DOM nodes that gives the last of the nodes contained
 * directly in the reference node.)
 *
 * @param sib  The reference node.
 * @return     Either:
 *               1) The last child of |sib| that is not
 *                  ignorable according to |is_ignorable|, or
 *               2) null if no such node exists.
 */
function last_child( par )
{
  var res=par.lastChild;
  while (res) {
    if (!is_ignorable(res)) return res;
    res = res.previousSibling;
  }
  return null;
}

/**
 * Version of |firstChild| that skips nodes that are entirely
 * whitespace and comments.
 *
 * @param sib  The reference node.
 * @return     Either:
 *               1) The first child of |sib| that is not
 *                  ignorable according to |is_ignorable|, or
 *               2) null if no such node exists.
 */
function first_child( par )
{
  var res=par.firstChild;
  while (res) {
    if (!is_ignorable(res)) return res;
    res = res.nextSibling;
  }
  return null;
}

/**
 * Version of |data| that doesn't include whitespace at the beginning
 * and end and normalizes all whitespace to a single space.  (Normally
 * |data| is a property of text nodes that gives the text of the node.)
 *
 * @param txt  The text node whose data should be returned
 * @return     A string giving the contents of the text node with
 *             whitespace collapsed.
 */
function data_of( txt )
{
  var data = txt.data;
  // Use ECMA-262 Edition 3 String and RegExp features
  data = data.replace(/[\t\n\r ]+/g, " ");
  if (data.charAt(0) == " ")
    data = data.substring(1, data.length);
  if (data.charAt(data.length - 1) == " ")
    data = data.substring(0, data.length - 1);
  return data;
}