var SCRIBEFIRE_DB = {
	db : null,
	inBatch : false,
	
	dbStack : 0,
	
	notesTable : "notes_v1",
	templatesTable : "blog_templates",
	stateTable : "state_v1",
	historyTable : "history_v1",
	blogsTable : "blogs_v1",
	
	dbFile : null,
	timer : null,
	
	load : function () {
		var db = this.getDB();
		
		if (!db.tableExists(this.templatesTable)) {
			db.executeSimpleSQL("CREATE TABLE "+this.templatesTable+" (blog_guid TEXT PRIMARY KEY, template TEXT, date INTEGER, live_preview INTEGER DEFAULT 1)");
		}
		
		if (!db.tableExists(this.notesTable)) {
			db.executeSimpleSQL("CREATE TABLE "+this.notesTable+" (id INTEGER PRIMARY KEY AUTOINCREMENT, title TEXT, content TEXT, modified INTEGER)");
		}
		
		if (!db.tableExists(this.stateTable)) {
			db.executeSimpleSQL("CREATE TABLE "+this.stateTable+" (id INTEGER PRIMARY KEY AUTOINCREMENT, type TEXT, entryId TEXT, originalDateCreated TEXT, content TEXT, title TEXT, tags TEXT, categories TEXT, isDraft INTEGER DEFAULT 0)");
		}
		
		if (!db.tableExists(this.historyTable)) {
			db.executeSimpleSQL("CREATE TABLE "+this.historyTable+" (id INTEGER PRIMARY KEY AUTOINCREMENT, blogId INTEGER DEFAULT 0, postId TEXT, content TEXT, title TEXT, categories TEXT, dateCreated INTEGER, tags TEXT, isDraft INTEGER DEFAULT 0, type TEXT DEFAULT 'post')");
		}
		
		if (!db.tableExists(this.blogsTable)) {
			db.executeSimpleSQL("CREATE TABLE "+this.blogsTable+" (id INTEGER PRIMARY KEY AUTOINCREMENT, name TEXT, type TEXT, api TEXT, key TEXT, username TEXT, guid TEXT, url TEXT, blogId TEXT, useBoolean TEXT)");
		}
		
		this.closeDB(db);
	},
	
	getDB : function () {
		this.dbStack++;
		
		if (this.dbStack > 1) {
			return this.db;
		}
		
		var storageService = Components.classes["@mozilla.org/storage/service;1"]
		                        .getService(Components.interfaces.mozIStorageService);
		
		if (!this.dbFile) {
			this.dbFile = Components.classes["@mozilla.org/file/directory_service;1"]
			                     .getService(Components.interfaces.nsIProperties)
			                     .get("ProfD", Components.interfaces.nsIFile);
			this.dbFile.append("scribefire.sqlite");
		}
		
		this.db = storageService.openDatabase(this.dbFile);
		
		return this.db;
	},
	
	closeDB : function (foo, force) {
		this.dbStack--;
		
		if (force) this.dbStack = 0;
	
		if (this.dbStack == 0) {
			try { this.db.close(); } catch (e) { }
		}
	},
	
	getBlogTemplate : function (blog_guid) {
		var db = this.getDB();
		var select = db.createStatement("SELECT template FROM "+SCRIBEFIRE_DB.templatesTable+" WHERE blog_guid=?1");
		select.bindStringParameter(0, blog_guid);
		
		var template = "";
		
		try {
			while (select.executeStep()) {
				template = select.getString(0);
				break;
			}
		} catch (e) {
		} finally {
			select.reset();
		}
		
		this.closeDB(db);
		
		return template;
	},
	
	saveBlogTemplate : function (blog_guid, template, live_preview) {
		var db = this.getDB();
		
		if (live_preview) live_preview = 1;
		else live_preview = 0;
		
		var insert = db.createStatement("INSERT INTO blog_templates (blog_guid, template, date, live_preview) VALUES (?1, ?2, ?3, ?4)");
		insert.bindUTF8StringParameter(0, blog_guid);
		insert.bindUTF8StringParameter(1, template);
		insert.bindInt64Parameter(2, (new Date().getTime()));
		insert.bindInt64Parameter(3, live_preview);
		
		try { insert.execute(); } catch (duplicateKey) {
			var update = db.createStatement("UPDATE blog_templates SET template=?1, date=?2 WHERE blog_guid=?3");
			update.bindUTF8StringParameter(0, template);
			update.bindInt64Parameter(1, (new Date().getTime()));
			update.bindUTF8StringParameter(2, blog_guid);
			
			try { update.execute(); } catch (e) { }
		}
		
		this.closeDB(db);
	},
	
	deleteBlogTemplate : function (blog_guid) {
		var db = this.getDB();
		var deleter = db.createStatement("DELETE FROM blog_templates WHERE blog_guid=?1");
		deleter.bindUTF8StringParameter(0, blog_guid);
		deleter.execute();
		this.closeDB(db);
	}
};

var SCRIBEFIRE_BLOG = {
	serviceObject : null,
	_guid : null,
	
	password : "",
	
	get guid() {
		return this._guid;
	},
	
	set guid(guid) {
		SFLOG("Setting guid to " + guid);
		
		this._guid = guid;
		
		try {
			this.setupServiceObject();
		} catch (e) {
			throw e;
		}
	},
	
	getter : function (field) {
		var db = SCRIBEFIRE_DB.getDB();
		
		var select = db.createStatement("SELECT "+field+" FROM "+SCRIBEFIRE_DB.blogsTable+" WHERE guid=?1 LIMIT 1");
		select.bindStringParameter(0, this.guid);
		
		var rv = "";
		
		try {
			while (select.executeStep()) {
				rv = select.getUTF8String(0);
			}
		} catch (e) {
			alert(e);
		} finally {
			select.reset();
		}
		
		SCRIBEFIRE_DB.closeDB(db);
		
		return rv;
	},
	
	setter : function (field, value) {
		var db = SCRIBEFIRE_DB.getDB();
		
		var update = db.createStatement("UPDATE "+SCRIBEFIRE_DB.blogsTable+" SET "+field+"=?1 WHERE guid=?2");
		update.bindUTF8StringParameter(0, value);
		update.bindStringParameter(1, this.guid);
		update.execute();
		
		SCRIBEFIRE_DB.closeDB(db);
	},
	
	get supportsTags() {
		if (typeof this.serviceObject.supportsTags != 'undefined') {
			return this.serviceObject.supportsTags;
		}
		
		return false;
	},
	
	get supportsPages() {
		if (typeof this.serviceObject.supportsPages != 'undefined') {
			return this.serviceObject.supportsPages;
		}
		
		return false;
	},
	
	get supportsDrafts() {
		if (typeof this.serviceObject.supportsDrafts != 'undefined') {
			return this.serviceObject.supportsDrafts;
		}
		
		return false;
	},
	
	get addCategories() {
		if (typeof this.serviceObject.addCategories != 'undefined') {
			return this.serviceObject.addCategories;
		}
		
		return false;
	},
	
	get createCategories() {
		if (typeof this.serviceObject.createCategories != 'undefined') {
			return this.serviceObject.createCategories;
		}
		
		return false;
	},

	get api() {
		return this.getter("api");
	},
	
	get type() {
		return this.getter("type");
	},
	
	get key() {
		return this.getter("key");
	},
	
	get blogId() {
		return this.getter("blogId");
	},
	
	get useBoolean() {
		return this.getter("useBoolean");
	},
	
	get username() {
		return this.getter("username");
	},
	
	get url() {
		return this.getter("url");
	},

	get name() {
		return this.getter("name");
	},
	
	get id() {
		var rv = this.getter("id");
		return parseInt(rv, 10);
	},
	
	set api(val) {
		this.setter("api", val);
	},
	
	set type(val) {
		this.setter("type", val);
	},
	
	set key(val) {
		this.setter("key", val);
	},
	
	set blogId(val) {
		this.setter("blogId", val);
	},
	
	set useBoolean(val) {
		this.setter("useBoolean", val);
	},
	
	set username(val) {
		this.setter("username", val);
	},
	
	set url(val) {
		this.setter("url", val);
	},
	
	set name(val) {
		this.setter("name", val);
	},
	
	setupServiceObject : function () {
		var apiObject = null;
		var aService = this.type;

		// Determine the API type from either the blogtype value of the blogapi value.
		switch (aService) {
			case "atom":
				apiObject = new perFormancingAtomAPI();
				break;
			case "s_atom":
			case "blogger_com":
			case "atom_blogger":
			case "atom_blogger_cust":
				apiObject = new perFormancingBloggerAtomAPI();
				break;
			case "livejournal_com":
			case "blogger_cust":
			case "s_blogger":
			case "blogger":
				apiObject = new perFormancingBloggerAPI();
				break;
			case "wordpress_com":
			case "typepad_com":
			case "wordpress_cust":
			case "moveabletype_cust":
			case "s_mt":
			case "mt":
			case "drupal_cust":
			case "textpattern_cust":
				apiObject = new perFormancingMovableTypeAPI();
				break;
			case "metaweblog":
			case "metaweblog_cust":
			case "s_metaweblog":
			case "msnspaces_com":
			case "performancing_com":
			case "jeeran_com":
			case "splinder_com":
			case "xanga_com":
			case "roller_cust":
				apiObject = new performancingMetaweblogAPI();
				break;
			case "s_myspace":
			case "myspace_com":
				apiObject = new performancingMySpaceAPI();
				break;
			case "s_tumblr":
			case "tumblr_com":
				apiObject = new performancingTumblrAPI();
				break;
			default:
				throw "Could not set up API object for blog of type " + aService;
				break;
		}
		
		var url = this.url;
		var username = this.username;
		
		var thePassword = gPerformancingUtil.goGetPassword(this.username, this.url);
		apiObject.doSetup();
		apiObject.init(this.type, this.key, this.api, this.blogId, null, this.username, thePassword, false);

		this.serviceObject = apiObject;
	}
};